package GameEngine.Core;
import GameEngine.GameMode.GameMode;
import GameEngine.GameMode.MeasureUpMode;

import java.io.IOException;
import java.net.ServerSocket;
import java.net.Socket;
import java.util.*;

/**
 * The server will be designed to host multi games at the same time. 
 * Each game runs on its own thread. A game will have 
 * - one master server to communicate with master client, 
 * - one game mode, 
 * - n mobile servers associated with n mobile clients, and  details later. 
 * Each game have some rounds and each round have some questions. 
 * When the game is playing it will go from the 1st round to the last round automatically. 
 * A game has multi stages and it can be considered as a state machine. 
 * A game will be terminated by the master client or the game engine. 
 * @author tranlongtien
 *
 */
public class Game implements Runnable{


	/**
	 * @uml.property  name="masterServer"
	 */
	private MasterCommunicator masterServer;
	/**
	 * @uml.property  name="clients"
	 */
	private ArrayList<MobileCommunicator> mobileClients;
	/** 
	 * @uml.property name="state"
	 */
	private int state;
	/**
	 * @uml.property  name="currentQues"
	 */
	private Question currentQues;
	/** 
	 * @uml.property name="gameMode"
	 */
	private GameMode gameMode;

	private Round currentRound;
	private int gameID;

	/**
	 * @uml.property  name="quesPerRound"
	 * number of questions on each round???
	 */
	private int quesPerRound;

	/** 
	 * @uml.property name="lectureID"
	 */
	private int lectureID = 1;


	/**
	 * @uml.property  name="totalPossibleScore"
	 */
	private String totalPossibleScore = "0";

	/** 
	 * Constructor: called by GE, when the master client connects to game engine
	 * Create new MasterServer
	 * Start the MasterServer Thread to receive messages from master client
	 * @throws IOException 
	 */
	public Game(Socket soc, int id) throws IOException{

		gameID =id;
		//Receiver thread to response to message from master client
		// Look more in the run() function of MasterServer class
		masterServer = new MasterCommunicator(soc,this);
		Thread masterReceiver = new Thread(masterServer);		
		state = LG.STATE_INIT; 
		masterReceiver.start(); //make onReceive of MasterServer run to receive message from master client
		mobileClients = new ArrayList<MobileCommunicator>();
	}


	/**
	 * To be added: find out the state the Game may be in???
	 * This function run at the same time with the receiver thread (onReceiver function) of MasterServer and MobileCommunicator
	 */
	public void run() {
/*		while (state != LG.STATE_EXIT_GAME) // run until game state is set to EXITGAME
		{
			try {
				Thread.sleep(1000);
			} catch (InterruptedException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			//the game will be put in this state when the master client send the start request
			if (state == LG.STATE_START_GAME) 
				playGame();
		}*/
	}

	/**
	 * This function implement the Game story as a state machine, changing the state of the game during the game life
	 * The game will be in the state called "Playing on round" most of the time
	 */
	private void playGame(){
		// Each game have some rounds, hOw to move between round? automaticlly, when reach end of round
		// On each round will go through alot of questions, thee move between question is automatically (after some seconds)
		currentRound = new Round(this.lectureID);

		state = LG.STATE_PLAYING_ON_ROUND;

		System.out.println("Start playing game!");

		while (true) {
			if (state == LG.STATE_PLAYING_ON_ROUND) {
				currentQues = currentRound.getNextQuestion();
				// if the question is not null then move to state LG.STATE_PUSH_QUESTION to send question.
				if (currentQues != null) {
					state = LG.STATE_PUSH_QUESTION;
				} else // if there is no more question in the round then set to
					// the STATE "end of round"
				{
					state = LG.STATE_END_OF_ROUND;
				}
			} else if (state == LG.STATE_PUSH_QUESTION) {
				// send question to master client
				String masQues = gameMode.createMasterQuestion(currentQues);
				System.out.println(masQues);
				
				this.masterServer.sendMessage(masQues);
				// send question to mobile clients ????
				String mobQues = gameMode.createClientQuestion(currentQues);
				if (mobileClients.size()>0)
					for (MobileCommunicator mobSrv:mobileClients){
						mobSrv.sendMessage(mobQues);
						mobSrv.updateResutl(); // set the answerIndex =-1
					}
				state = LG.STATE_WAITING_REPLY;
			} else if (state == LG.STATE_WAITING_REPLY) {
				// wait for the answers from mobile clients and then send result to master client to display then go back to "Playing on round" state to continue
				totalPossibleScore += gameMode.getMaxScore();
				//Sleep to wait
				Thread.yield();
				try {
					Thread.sleep(currentQues.getTimeLimit() * 1000);
				} catch (InterruptedException e) {
					e.printStackTrace();
				}               
				state =LG.STATE_DISPLAY_ANSWER;
			}else if (state ==LG.STATE_DISPLAY_ANSWER){
				//	have a break between questions because
				Thread.yield();
				try {
					Thread.sleep(LG.DELAY_AFTER_SENDING_QUESTION);
				} catch (InterruptedException e) {
					e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
				}

				// send result to master client
				String res1 = gameMode.handleQuestionReply(this.mobileClients);
				masterServer.sendMessage(res1);
				
				
				for (MobileCommunicator c:mobileClients){
					String res2 = gameMode.handleClientQuestionReply(c, currentQues);
					//System.out.println("send reslt to client " + res2);
					c.sendMessage(res2);
				}

				// Wait some time be4 starting other quesiton
				Thread.yield();
				try {
					Thread.sleep(LG.DURATION__DISPLAY_QUES_RESULT);
				} catch (InterruptedException e) {
					e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
				}
				// go back to "Playing on round" state, repeat to send other question
				state = LG.STATE_PLAYING_ON_ROUND;
				
			}else if (state == LG.STATE_END_OF_ROUND) {
				//handle end of round
				String res2 = gameMode.handleEndOfRound(mobileClients);
				masterServer.sendMessage(res2);				
//				try {
//					Thread.sleep(2000);
//				} catch (InterruptedException e) {
//					// TODO Auto-generated catch block
//					e.printStackTrace();
//				}
				// if there are  more rounds then prepare the data and then set the state to "Playing on round" to play on the next round
				// if there are no more round then go the "End of game" state
				if (currentRound.nextRound())
					state = LG.STATE_PLAYING_ON_ROUND;
				else
					state = LG.STATE_END_OF_GAME;
			} else if (state == LG.STATE_END_OF_GAME) {
				//handle end of game
				//masterServer.sendMessage(Message.createEndGameMessage());
				for (MobileCommunicator c: mobileClients){
					c.sendMessage(Message.createEndGame4Mobile(c));
				}
				break;
			} else break;
		}
	}

	/**
	 * this function do appropriate things when a message arrive
	 * @param msg: of type Message, is an object then encapsulated the message string receive from client
	 */
	public void processMasterMessage(Message msg)
	{
		switch (msg.getTypeOfMsg())
		{
		case Message.MSG_NUMBER_OF_QUES:
			if (state == LG.STATE_INIT)
			{
				int numQues = Integer.parseInt(msg.getMsgContent());
				quesPerRound = numQues;
			}
			break;
		case Message.MSG_GAME_MODE:
			if (msg.getMsgContent().equals(String.valueOf(LG.GAMEMODE_BEAT_THE_SCORE)))
				gameMode = new MeasureUpMode();
			//other mode to be added here
			break;
		case Message.MSG_EXIT: // set the game state to EXITGAME 
			state =LG.STATE_EXIT_GAME; // should have some concurrent control here
			break;
		case Message.MSG_START:
			//state =LG.STATE_START_GAME;
			playGame();
			break;
		case Message.MSG_LECTURE_ID:
			lectureID = Integer.parseInt(msg.getMsgContent());			
			break;
		case Message.MSG_NEXT_QUESTION:
			state= LG.STATE_PLAYING_ON_ROUND;
			break;
		}		
	}

	/**
	 * this function do appropriate things when a message arrive
	 * @param msg: of type Message, is an object then encapsulated the message string receive from client
	 */
	public void processMobileMessage(Message msg, MobileCommunicator c){
		//System.out.println("response to mobile msg from game!");
		msg.printMsg();
		if (msg.isAnswer()){
			boolean isCorrect = currentQues.getCorrectAlternative() == msg.getAnswerIndex();			
			c.updateResult(isCorrect, msg.getAnswerIndex());			
		}
	}

	/**
	 * 
	 * @param socket
	 */
	public void addMobileCommunicator(Socket socket, String user, String pass, String lecturecode){
		//Create new MobileCommunicator obj to receive message from mobile client
		MobileCommunicator MobileCommunicator;
		try {
			MobileCommunicator = new MobileCommunicator(socket, this);
			MobileCommunicator.setUserName(user);
			MobileCommunicator.setPassword(pass);
			MobileCommunicator.setLectureCode(lecturecode);
			
			MobileCommunicator.resetResult();			
			mobileClients.add(MobileCommunicator);
			Thread mobileReceiveThd = new Thread(MobileCommunicator);
			mobileReceiveThd.start(); // make onReceive of MobileCommunicator run to receive message from mobile client
			// Send report to game master
			masterServer.sendMessage(Message.createMobileLoginMsg(user));
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}		
	}//

	/**
	 * Exit the game
	 * Set game state to exit 
	 * End all connections to master and mobilet clients.
	 *
	 */
	public void killGame()
	{
		try{
			System.out.println("End connections!");
			state = LG.STATE_EXIT_GAME;
			masterServer.closeConnection();
			if (!mobileClients.isEmpty())
				for (MobileCommunicator mobSrv: mobileClients)
				{
					mobSrv.closeConnection();
				}
		}
		catch(Exception e){

		}
	}//

	/**
	 * check if game is still alive (state <> LG.STATE_EXIT_GAME)
	 */
	public boolean isGameAlive()
	{
		if (state == LG.STATE_EXIT_GAME)
			return false;
		return true;			
	}//
	
	public MasterCommunicator getMasterServer(){
		return masterServer;
	}//

}// End class
