package GameEngine.Core;
import java.io.*;
import java.io.IOException;
import java.net.ServerSocket;
import java.net.Socket;

/**
 * - This class is responsible for managing network communication between the game and the clients (master and mobile)
 * it has function for send message and get messages. 
 * - The get message will run on the own thread, so that the message from client can be processed right
 * after it arrives.
 * - This class wil be extended for each type of client
 * @author tranlongtien
 *
 */
public class GameCommunicator implements Runnable{


	/**
	 * @uml.property  name="socket"
	 */
	protected Socket socket;
	/**
	 * @uml.property  name="in"
	 */
	protected BufferedReader in;
	/**
	 * @uml.property  name="out"
	 */
	protected PrintWriter out;
	/**
	 * @uml.property  name="myGame"
	 */
	protected Game myGame;

	/**
	 * @uml.property  name="messageIn"
	 * the message from client
	 */
	protected String strMessageIn = "";

	/**
	 * Represent message from client
	 */
	protected Message msg;
	
	/**
	 * Constructor
	 * @param socket
	 * @param game
	 * @throws IOException 
	 */
	public GameCommunicator(Socket socket, Game game) throws IOException{

		this.socket = socket;
		this.myGame = game;

		out = new PrintWriter(socket.getOutputStream());
		in = new BufferedReader(new InputStreamReader (socket.getInputStream()));

	}

	public void run() {
		// TODO Auto-generated method stub
		// if there is an error of receiveing message, the onReceive will set
		// the keepOnReceiving flag to false, make chance to exit while loop.
		boolean keepOnReceiving =true;
		
		while(keepOnReceiving)
			keepOnReceiving = onReceive();
		System.out.println("End receiver thread!");
	}


	/**
	 * This function response to messages from client, will be implemented by the extended class.
	 */
	public boolean onReceive(){
		return true;
	}


	/**
	 * Send a string message to client
	 * 
	 */
	public void sendMessage(Object messageOut){
		out.println(messageOut.toString());
		out.flush();		
	}

	/**
	 * receive a string message from client
	 * @throws IOException 
	 * 
	 */
	public String receiveStrMessage() throws IOException{
		String retStr ="";
		retStr = in.readLine();
		return retStr;
	}

	
	public void closeConnection() {
		try {
			in =null;
			out = null;
			socket.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	


}
