package GameEngine.Core;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.net.ServerSocket;
import java.net.Socket;

/**
 * Game manager is the 1st thread to run. The main responsibility of GE class is to: -	Set up environment -	Listen for connections from mobile and master clients on 2 additional saperated threads -	Authenticate clients -	Manage games created GE will base on 2 others classes to listen for 2 kinds of clients: mobile client that student uses to play game and master client that teacher uses.  They are MobileListener and MasterListener.  Since these 2 classes are only used by GE, they will be nested in the GE.  Additionally, they will be started in separated threads. so that messages from mobile and master client  can be processed in the same time
 * @uml.dependency   supplier="GameEngine.Core.GameList" stereotypes="Standard::Instantiate"
 */
public class GameManager {	
	/**
	 * @uml.property   name="masterListener"
	 */
	private GameListener masterListener;

	/**
	 * @uml.property   name="mobileListener"
	 */
	private GameListener mobileListener;

	/**
	 * @uml.property  name="games"
	 */
	private GameList games;
	
	public Configuration myConf;// = new Configuration();

	/** 
	 * Constructor
	 * set up environment, initiate variables.
	 */
	public GameManager(){
		//Create the game list
		games= new GameList();
		myConf = new Configuration();

		//Create the 2 listener
		this.mobileListener = new MobileListener(myConf.getMobileListenPort());
		this.masterListener = new MasterListener (myConf.getMasterListenPort());
	}

	/**
	 * -Set up environment
	 * -To create listenning threads
	 * 
	 */
	private void startListenning(){

		//Start the listening threads
		System.out.println("Start listening for connection from Master clients");
		listen4Masters();
		System.out.println("Start listening for connection from Mobile clients");
		listen4Mobiles();
		//
	}


	/**
	 * -This will create new thread to listen for mobile clients
	 * -The thread to be created will execute the run function of the MobileListener class, this class have a varibale called listener of type SocketServer; listener.accept() function call will wait for mobile client to connect to and return a Socket that will be use to communicate with the mobile client.
	 * 
	 */
	private void listen4Mobiles(){

		Thread mobListenerThrd = new Thread(mobileListener);
		mobListenerThrd.start();	
	}

	/** 
	 * - This will create new thread to listen for master clients.
	 * - Similar to the listen4Mobiles, The thread to be created will execute the run function of the MaterListener class, this class have a varibale called listener of type SocketServer; listener.accept() function call will wait for mobile client to connect to and return a Socket that will be use to communicate with the master client.
	 * - Note that the run function, the Socket return will be used to create new game. The constructor of the Game class have one parametter of Socket type. But the game will not be create directly, it will be create inside the games.add(Socket) function (Detail later)
	 */
	private void listen4Masters(){
		Thread masListenerThrd = new Thread(masterListener);
		masListenerThrd.start();
	}

	/** 
	 * System.out.println("GameEngine is running");
	 */
	public static void main(String[] agrs){
		System.out.println("GameEngine is running");
		GameManager myGameManager = new GameManager();
		myGameManager.startListenning();					
	}

	/** 
	 * Decode the input string to receive the original value, currently this function just return the strin with out doing anything.
	 */
	public String decode(String inStr){
		return inStr;
	}

	/**
	 * Encode the input string, current version just return the string without any modification.
	 */
	public String encode(String inStr){

		return inStr;
	}
////////////////////////////////////////////////////////////////////////////////////////////////////////
	private abstract class GameListener implements Runnable{
		
		protected ServerSocket listener;

		protected BufferedReader in;

		protected PrintWriter out;
		
		/** 
		 * The listenning thread, response when there is a connection
		 */
		public void run(){

			// TODO Auto-generated method stub
			while(true)
			{
				onConnection();
			}
		}

		/** 
		 * //constructor:
		 * //initiate listener variable
		 */
		public GameListener(int portNumber){

			try {
				listener = new ServerSocket(portNumber);
			} catch (IOException e) {
				System.out.println("Can't create listening socket on port " + portNumber);
				System.out.println("Game engine quits!");
				System.exit(1); // this is a critical problems, so need to exit the system.
			}
					
		}

		/** 
		 * // wait and do any stuff needed when a client connect
		 * this function will be extended by MobileListener and MasterListener class
		 */
		public abstract void onConnection();


		/**
		 * Now it alway return true, later it will look up the DB to check.
		 */
		public boolean checkUser(String userName, String password){
			return true;
		}

		/**
		 * Send a string message to client
		 * 
		 */
		public void sendMessage(Object messageOut){
			out.println(messageOut.toString());
			out.flush();
		}

		/**
		 * receive an integer from client
		 */
		public int receiveInteger() throws IOException {
			int i=-1;
			i = Integer.parseInt(in.readLine());
			return i;
		}

		/**
		 * @throws IOException 
		 */
		public String receiveString() throws IOException{
			String ret="";
			ret = in.readLine();
			return ret;

		}
	} //End GameListener class
	
	/////////////////////////////////////////////////////////////////////////////////////////////////////////
	
	private class MobileListener extends GameListener {

		/** 
		 * // The protocol between the client and GE should be more specific
		 * //wait and accept connection
		 * // authenticate users
		 * //Find out the game i to join
		 * // call to games(i).clients.addClient(Socket)
		 */
		public void onConnection(){			
			Socket mobServer;
			try {
				mobServer = this.listener.accept();
				out = new PrintWriter(mobServer.getOutputStream(),true);
				in = new BufferedReader(new InputStreamReader(mobServer.getInputStream()));

				//Authenticate user
				String user = "", pass="", lecCode = "";
				//Get user and pass sent from the mobile client
				user = receiveString();
				pass =receiveString();
				System.out.println("login msg: "+ user+pass);
				
				if (!checkUser(user, pass))	{
					System.out.println("Incorrect user name or password!");
					return;
				}
				
				// loop until at least one master client login
				// cause conflict?
				while (true){
					System.out.println("Mobile client wait for master connecting!");
					try {
						Thread.sleep(1000);
					} catch (InterruptedException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					}
					if (games.gameList.size() >0)
						break;					
				}								
				// Make sure to get the correct index of the choose game in the list from user								
				int myGameIndex = -1;
				lecCode = receiveString();
				System.out.println("lecode: " + lecCode);
				while(true){					
					try{
						myGameIndex= Integer.parseInt(lecCode.trim()); // can cause exception.
						if (myGameIndex  >=0 && myGameIndex < games.gameList.size()) // correct, then exit the loop
							break;																		
					} catch (Exception e){						
					}					
					sendMessage("" + Message.MSG_UNKNOWN_MESSAGE); // tell mobile the lecture code is not correct
					lecCode = receiveString(); // receive the data again 
				} 
				// say to client that lecture code is ok now!
				sendMessage("" + Message.MSG_LECTURE_ID);								
				
				// Add a mobile server to the game. This server responsible for receive msg from mobile
				// The response to the message will be handled by function processMobileMessage of Game class
				Game myGame = games.gameList.get(myGameIndex);
				myGame.addMobileCommunicator(mobServer,user, pass, lecCode);	
				// send login ok to mobile
				sendMessage(Message.createLoginOk());

			} catch (IOException e) {
				System.out.println("Error "+ e.getMessage());
				return; // no need to call System.exit(1);
			}		
		}//

		/**
		 * constructor
		 */
		public MobileListener(int portNumber){
			super(portNumber);
		}

	} // End MobileListener
	
///////////////////////////////////////////////////////////////////////////////////////////////////////

	/** 
	 * This class runs on its own thread and is responsible for listenning to connection from master clients.  This is the nested class in the GE so it has access to all member variables of GE event the private member. It will use the games (belong toGameList class) member to create a new game when there is connection from the master client. 
	 * Master listener is also responsible for cheking the user name and password of master client before letting client communicate with GE.
	 */
	private class MasterListener extends GameListener {

		/** 
		 * Constructor
		 */
		public MasterListener(int portNumber){
			super(portNumber);

		}

		/** 
		 * // The protocol between the client and GE, override the function on parent class
		 * //wait and accept connection
		 * // authenticate users
		 * //Create new game
		 * // call to games.addGame (Socket)
		 */
		public void onConnection(){

			Socket masterSocket; // The socket used to communicate with the master client
			String messageIn; // a message received from the client
			String user ="", pass="";

			//Wait and accept connection from master client
			try {
				//This socket will be used to communicate with the master.
				masterSocket=listener.accept();
				out = new PrintWriter(masterSocket.getOutputStream(),true);
				in = new BufferedReader(new InputStreamReader(masterSocket.getInputStream()));
						
				// send handshake to the master client
				sendMessage(Message.HANDSHAKE_MASTER); 
				// read handshake from client
				messageIn = receiveString(); 
				if (! messageIn.endsWith(Message.HANDSHAKE_MASTER))
				{
					System.out.println("Connected program is not master server!");
					return;
				}

				// Authenticate user
				// Get the username and password from the client
				messageIn = receiveString();
				user = decode(messageIn);
				messageIn = receiveString();
				pass = decode(messageIn);

				//look up user in the DB:
				if (!checkUser(user, pass))
				{
					System.out.println("Incorrect user name or password!");
					return ;
				}

				//Send login result
				sendMessage(LG.ERROR_LOGIN_OK);
				System.out.println("Maste Cliet logged in ok");
				
				//Every thing is ok now, add new game
				int id;
				try{
					id = games.addGame(masterSocket);
				} catch (Exception ex){
					id =-1;
				}
				
				// send game id to master
				sendMessage(id);
				System.out.println("New game is created ok");
				
			} catch (IOException e) {
				System.out.println("Error initiate game, " + e.getMessage());
				return;
			}
		}

	} // End of MasterListener
} // end of GameEngine
