package GameEngine.DataBase;
import GameEngine.Core.*;

import java.sql.*;

import com.mysql.jdbc.exceptions.MySQLIntegrityConstraintViolationException;


/**
 * Created by IntelliJ IDEA.
 * User: olekrst
 * Date: 26.jan.2007
 * Time: 12:06:26
 * This class connects to the SQL database. Its primary functionallity is to extract and register new users, and extracting
 * different round and the questions associated with them.
 * @see LectureGame.Core.Question
 */
public class DataBaseConnection {

    private ResultSet rs;
    private Statement stmt;

    private PreparedStatement getQuestion;
    private PreparedStatement newUser;
    private PreparedStatement countQuestionsInRound;
    private PreparedStatement getAllLectures;
    private PreparedStatement getAllRoundsInLecture;
    private PreparedStatement getGameMode;
    
    private Configuration myConf = new Configuration();

    /**
     * Creates a new database connection using com.mysql.jdbc.Driver
     */
    public DataBaseConnection() {

        stmt = null;
        try {
            Class.forName("com.mysql.jdbc.Driver");
            java.sql.Connection con = DriverManager.getConnection(LG.MYSQL_ADDRESS, myConf.getUserName(), myConf.getPassword());


            stmt = con.createStatement(ResultSet.TYPE_SCROLL_INSENSITIVE, ResultSet.CONCUR_UPDATABLE);

            // different queries for extracting and inserting info into database. Variable names sum up what the different
            // queries are.
            newUser = con.prepareStatement("INSERT INTO player VALUES(?,?,?,null,?);");
            getQuestion = con.prepareStatement("SELECT question,alternative,isCorrectAlt,timelimit FROM question,question_alternative WHERE question.questionID = ? AND question.questionID = question_alternative.questionID;");
            getAllLectures = con.prepareStatement("SELECT course,description,lectureid FROM lecture;");
            getAllRoundsInLecture = con.prepareStatement("SELECT roundid FROM round WHERE lecture=?;");
            countQuestionsInRound = con.prepareStatement("SELECT questionID FROM question q INNER JOIN round r ON q.roundid=r.roundid WHERE r.roundid=?;");
            getGameMode = con.prepareStatement("SELECT gametype FROM round WHERE roundid = ?;");
            rs = null;
            


        } catch (SQLException e) {
            e.printStackTrace();
        } catch (ClassNotFoundException e) {
            e.printStackTrace();
        }

    }

    /**
     * Returns the gamemode for a given round
     * @param roundID the rund to find the gamemode for
     * @return the gamemode code (see LG source for constants)
     * @see LectureGame.Core.LG
     */
    public int getGamgeMode(int roundID) {
        try {
            getGameMode.setInt(1, roundID);
            rs = getGameMode.executeQuery();
            if (rs.next()) return Integer.parseInt(rs.getString(1));

        } catch (SQLException e) {
            e.printStackTrace();
        }
        return -1;

    }

    /**
     * Extracts all the existing Lectures from the database. Formats the return String for sending to the masterclient
     *
     * @return a string containing all lectures, formatted for sending to the masterclient
     */
    public String getAllLectures() {
        try {
            rs = getAllLectures.executeQuery();
            StringBuilder buffer = new StringBuilder();

            // number of courses|course code|lecture description|lectureID| ... |
            // example:
            // @^LGN5|TDT5555|test course 1|001|TDT4900|Masterprosjekt|002|TDT1234|Counting for non-counters|003|TDT4455|Numerics|004|TDT9999|The best course in the world|005^@");

            int lectureCounter = 0;
            while (rs.next()) {
                buffer.append("|");
                buffer.append(rs.getString(1));
                buffer.append("|");
                buffer.append(rs.getString(2));
                buffer.append("|");
                buffer.append(rs.getString(3));
                lectureCounter++;
            }
            buffer.insert(0, Integer.toString(lectureCounter));

            return buffer.toString();
        } catch (SQLException e) {
            e.printStackTrace();
        }
        return null;
    }

    /**
     * <b>NOT IMPLEMENTED</b>
     *
     * Method to be implemented when a Question editor is added
     *
     * @param question the question to add to the database
     * @param lectureId the lectureID of the lecture which will contain the question
     */
    public void addQuestion(Question question, String lectureId) {
        // add handeling code here
    }



    /**
     * Method for checking username and password. Returns one of the following error codes:
     *
     * - ERROR_USER_EXIST_WRONG_PW;
     * - ERROR_LOGIN_OK
     * - ERROR_UNKNOWN_USER
     *
     * @see LectureGame.Core.LG
     *
     * @param usr
     * @param password
     * @return error code as defined in LG
     */
    public int getUser(String usr, String password) {

        String username;

        if(usr.equals(""))return LG.ERROR_USER_EXIST_WRONG_PW;
        username = usr;

        try {
            rs = stmt.executeQuery("SELECT username,password FROM player WHERE username = '" + username + "';");

            if (rs.next()) {
                String pw = rs.getString("password");

                if (pw.equals(password)) {

                    ConsoleWriter.debug("Riktig! " + username + " sitt passord er: " + password,3);
                    return LG.ERROR_LOGIN_OK;

                } else {
                    ConsoleWriter.printErr("Feil passord for "+username);
                    return LG.ERROR_USER_EXIST_WRONG_PW;
                }
            }
            ConsoleWriter.printErr("no user by the name of '"+username+"'");
            return LG.ERROR_UNKNOWN_USER;
        } catch (SQLException e) {
            e.printStackTrace();
            
            return LG.ERROR_UNKNOWN_USER;
        } catch (Exception e){
            return LG.ERROR_UNKNOWN_USER;
        }

    }

    /**
     * Method for creating new users in the database. Returns one of the following error codes:
     *
     * - ERROR_LOGIN_OK
     * - ERROR_USERNAME_ALLREADY_EXISTS
     *
     * @see LectureGame.Core.LG

     * @param usr the username
     * @param password the password (cannot be "")
     * @return an error code, as defined in LG constants
     */
    public int newUser(String usr, String password) {

        String username;

        if(usr.equals("") || usr==null) return LG.ERROR_USERNAME_ALLREADY_EXISTS;
        else username = usr;

        try {


            newUser.setString(1, username);
            newUser.setString(2, password);
            newUser.setDate(3, new Date(System.currentTimeMillis()));
            newUser.setBoolean(4,false);
            //newUser.executeQuery();
            newUser.executeUpdate();

            return LG.ERROR_LOGIN_OK;
        } catch (MySQLIntegrityConstraintViolationException e) {
            return LG.ERROR_USERNAME_ALLREADY_EXISTS;
        }
        catch (SQLException e) {
            e.printStackTrace();
            return LG.ERROR_USERNAME_ALLREADY_EXISTS;
        }
    }

    /**
     * Extracts all the rounds in a given lecture. Returns the roundIDs of the rounds
     * @param lecture
     * @return the roundID from the database of the round
     */
    public int[] getRoundsInLecture(int lecture) {

        try {
            getAllRoundsInLecture.setInt(1, lecture);
            ConsoleWriter.printErr(getAllRoundsInLecture.toString());
            rs = getAllRoundsInLecture.executeQuery();
            rs.last();
            int returnvalues[] = new int[rs.getRow()];
            rs.first();

            int i = 0;

            do {
                returnvalues[i] = rs.getInt(1);
                i++;
            } while (rs.next());

            return returnvalues;


        } catch (SQLException e) {
            e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
        }

        return null;
    }

    /**
     * Extracts an array containing all the questions in a lecure
     * @param roundID
     * @return Question[] with all lectures in the requested round
     * @see LectureGame.Core.Question
     */
    public Question[] getQuestion(int roundID) {
        //SELECT course,question,alternative,isCorrectAlt FROM lecture,question,question_alternative WHERE lecture.roundID = question.roundID AND question.questionID = question_alternative.questionID;
        try {
            countQuestionsInRound.setInt(1, roundID);
            ConsoleWriter.debug("Number of Questions in round "+roundID+" "+countQuestionsInRound.toString(),3);

            rs = countQuestionsInRound.executeQuery();

            rs.last();
            int numberOfQuestions = rs.getRow();
            rs.first();

            ConsoleWriter.debug("number of questions: " + numberOfQuestions,3);

            int questionsInRound[] = new int[numberOfQuestions];

            int temp = 0;
            do {
                questionsInRound[temp] = rs.getInt(1);
                temp++;
            } while (rs.next());

            Question returnQuestions[] = new Question[numberOfQuestions];

            for (int i = 0; i < numberOfQuestions; i++) {
                getQuestion.setInt(1, questionsInRound[i]);
                rs = getQuestion.executeQuery();
                rs.last();
                int length = rs.getRow();
                rs.first();

                if(length>=6){
                    System.out.println("Question ID: " + questionsInRound[i]+ " har for mange spm");
                    length=6;
                }

                String question;
                String alternatives[] = new String[length];
                String timelimit;
                int correctAlt = -1;

                try{


                question = rs.getString(1);
                    }catch(java.sql.SQLException e){
                    System.out.println("Sprsml "+questionsInRound[i]+" har trbbel ("+i+" 'te iterasjon");
                    question = "";
                }


                int counter = 0;

                alternatives[counter] = rs.getString(2);
                if (rs.getInt(3) == 1) correctAlt = counter;

                timelimit = rs.getString(4);
                if (timelimit.equals("0") || timelimit.equals("")) timelimit = LG.DEFAULT_QUESTION_TIMELIMIT;

                counter++;

                while (rs.next()&& counter<6) {
                    alternatives[counter] = rs.getString(2);
                    if (rs.getInt(3) == 1) correctAlt = counter;
                    counter++;
                }

                returnQuestions[i] = new Question(question, counter, alternatives, timelimit, correctAlt);
            }
            return returnQuestions;

        } catch (SQLException e) {
            e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
            return null;
        }
    }
}


