package GameEngine.GameMode;
import java.util.ArrayList;

import GameEngine.Core.GameCommunicator;
import GameEngine.Core.MobileCommunicator;
import GameEngine.Core.Question;
import GameEngine.DataBase.DataBaseConnection;


//import LectureGame.Core.StudentClient;

/**
 * Created by IntelliJ IDEA.
 * User: olekrst
 * Date: 22.feb.2007
 * Time: 13:19:13
 * Modified by Tien April 2008
 * Interface used to dynamically swap the gameMode. All GameModes must implement this interface.
 * The classes that implement this interface is responsible for package (create) the message that is exchanged between game engine and
 * game client.
 */
public interface GameMode {

    /**
     * Method for creating the reply string which is sendt to the masterclient.
     * This string contains the score of all active student clients, and is sendt from the server after each
     * question has timed out.
     *
     * @return a string formatted for sending to the master client, containing the score of all the active student clients
     */
    String handleQuestionReply(ArrayList<MobileCommunicator> clients);

    /**
     * Formats an end of round string for sending to the masterclient
     *
     * @return a formatted string for sending to the master client
     */
    String handleEndOfRound(ArrayList<MobileCommunicator> clients);

    /**
     * Formats a question for sending to the master client.
     *
     * @return a string formatted and ready for sending to the master client
     */
    String createMasterQuestion(Question ques);

    /**
     * Uses the current question in the round and formats it for sending to the student clients
     * @return correctly formatted string containing the question
     * @see LectureGame.Core.StudentClient
     * @see Question
     */
    String createClientQuestion(Question ques);

    /**
     * Formats the results of the question answers from the studentclients
     * @param score the score received from answering the last question
     * @param roundScore total score in the current round
     * @param totalPossibleScore overall score
     * @return a sting correctly formatted for sending to the student clients
     */
    String handleClientQuestionReply(MobileCommunicator c, Question currentQuestion);

    /**
     * Returns the maximum possible score per question
     * @return max score per question
     * @see Question
     */
    int getMaxScore();


    /**
     * Checks the answer from the studentclients and returns an int depending on what is expected from the gamemode.
     * Typically this method will return 1 for correct and 0 for false.
     * @param answer the answer index represented by a string
     * @return an integer (typically 1 for correct, and 0 for false)
     */
    boolean checkAns(int answer,Question currentQuestion);


	/**
	 * @uml.property  name="maxScorePerQuestion"
	 */
	public static final int maxScorePerQuestion = 1;

}
