package Masterclient.Graphics;

import java.awt.Button;
import java.awt.Color;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.GradientPaint;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.GridLayout;
import java.awt.Insets;
import java.awt.Label;
import java.awt.LayoutManager;
import java.awt.LayoutManager2;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JPanel;
import javax.swing.JTextArea;
//import javax.swing.Timer;
import java.util.Timer;
import java.util.TimerTask;

import Masterclient.core.Question;

public class QuestionPanel extends JPanel {

	private ClockWise clockPanel;
	private TextPanel textPanel;

	
	public QuestionPanel(){
		super();

		clockPanel = new ClockWise();
		textPanel = new TextPanel();
		initQuesPanel();
	}

	private void initQuesPanel(){
		//this.setBackground(Color.BLACK);
		//LayoutManager myLayout = new BoxLayout(this, BoxLayout.Y_AXIS);
		LayoutManager myLayout = new GridLayout(2,1);
		this.setLayout(myLayout);
		this.add(clockPanel);
		this.add(textPanel);
	}

	public void displayQuestion(Question q){
		clockPanel.displayQuestion(q);
		textPanel.displayQuestion(q);
	}

	/////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	/**
	 * 
	 */
	public class ClockWise extends JPanel implements Runnable //,ActionListener
	{	
		private int dia ; // the diameter of the circle
		private int rx, ry; // upper left point of bounded square
		private Timer timer ; 
		long startTime =0, endTime = 0;
		Thread paintThread;
		private int currentTick =0; // each tick is each time the timer triger, this variable record how many tick have passed
		int totalTicks; // how many "ticks" in total, "tick" is each time the timer triger

		private int angleDraw = 0;
		private boolean redrawTimeString = true;
		private boolean redrawClock =true;

		private int quesDuration; // how long to answer question in seconds
		int timeLeft; // how long you have left to answer, will be cout down for example: 15,14, 13, ...,1,0
		private final int SLEEP_TIME  =10; //timer triger every 10 miliseconds

		private int timeStrUpdateCycle ; // update count down time every 1 second (in this case ~ 100 ticks of timer)
		private int clockUpdateCycle = timeStrUpdateCycle/6; //time to redraw the picture of count down clock: 6 times slower then the speed of update time string
		private int angleUnit; // the unit of arc update at one time

		private SoPlayer playerClick; // to play sound
		private SoPlayer playerEndQues;

		/**
		 * contrtructor
		 *
		 */
		public ClockWise(){
			super();
			playerClick = new SoPlayer("resources/click.wav");	
			playerEndQues = new SoPlayer("resources/clock.wav");		
		}

		public void displayQuestion(Question q){						
			quesDuration = q.getTimeLimit(); // in seconds
			currentTick =0; //number of 10 miliseconds (1/100 sec) has passed.
			totalTicks = quesDuration* 1000/SLEEP_TIME; // how many "ticks" in total, "tick" is each time the timer triger
			timeLeft = quesDuration;

			angleDraw = 0;			

			redrawTimeString = true;
			redrawClock = true;

			timeStrUpdateCycle = 1000/SLEEP_TIME; // update count down time every 1 second (in this case ~ 100 ticks of timer)
			clockUpdateCycle = timeStrUpdateCycle/3; //3 times slower then the speed of update time string
			angleUnit = 360/quesDuration/3; // the unit of arc update at one time
			//Start counting the "tick"
			timer= new Timer(); // java.util.Timer;
			//timer.schedule(new MyTimer(),0, SLEEP_TIME);
			timer.scheduleAtFixedRate(new MyTimer(),0, SLEEP_TIME);

			//Swing timer
			//timer= new Timer(SLEEP_TIME, this);
			//timer.start();

			startTime = System.currentTimeMillis();
			// GUI update thread
			paintThread= new Thread(this);
			paintThread.start();
		}		

		@Override
		protected void paintComponent(Graphics g) {
			// TODO Auto-generated method stub
			super.paintComponent(g);
			int w = getWidth();     // get width of panel.
			int h = getHeight();    // get its height.
			dia = h/2;
			rx = (w/2) - (dia/2);
			ry = (h/2) - (dia/2);		      		  
			this.setBackground(Color.BLACK);
			drawClock(g,angleDraw);				
			drawTimeString(g, Integer.toString(timeLeft));				
		}

		/**
		 * draw the "picture" of the count down clock!
		 * @param g
		 * @param radAngle
		 */
		private void drawClock(Graphics g, int radAngle)	{				
			Graphics2D g2 = (Graphics2D) g;
			// Gradient from the center to the most right point
			GradientPaint gp;
//			gp = new GradientPaint(rx + dia/2,ry +dia/2,  new Color(0,0,1.0F,1.0F), rx+dia ,ry + dia/2, Color.WHITE, true);
//			g2.setPaint(gp);
//			g2.fillArc(rx, ry, dia, dia, 0, radAngle);
			// gradient from center to the bottum point of the circle!
			gp = new GradientPaint(rx + dia/2,ry +dia/2, new Color(0,0,1.0F,0.3F), rx +dia/2, ry+dia, Color.WHITE, true);
			g2.setPaint(gp);
			g2.fillArc(rx, ry, dia, dia, 0, radAngle);	

			if (timeLeft == 0){ // make sure that the whole circle is drawn
//				gp = new GradientPaint(rx + dia/2,ry +dia/2,  new Color(0,0,1.0F,0.8F), rx+dia ,ry + dia/2, Color.WHITE, true);
//				g2.setPaint(gp);
//				g2.fillArc(rx, ry, dia, dia, 0, 360);
				// gradient from center to the bottum point of the circle!
				gp = new GradientPaint(rx + dia/2,ry +dia/2, new Color(0,0,1.0F,0.3F), rx +dia/2, ry+dia, Color.WHITE, true);
				g2.setPaint(gp);
				g2.fillArc(rx, ry, dia, dia, 0, 360);	
			}			
		}

		/**
		 * Draw the count down time for a question (15, 14, 13, ...0)
		 * @param g
		 * @param str
		 */
		private void drawTimeString(Graphics g, String str)	{
			Font font = new Font("Sanserif", Font.BOLD,dia/3*2);
			g.setColor(new Color(0.9F,0.8F,0.1F,0.6F));
			g.setFont(font);
			FontMetrics fm = g.getFontMetrics(font);
			g.drawString(str, rx + dia/2 - fm.stringWidth(str)/2, ry + dia/2 - fm.getHeight()/2 + fm.getAscent());
		}

		/**
		 * this function will be fired when the timer triger every 10 milliseconds
		 * it is responsible for update the time string (15,14,13,...0) and the area of the clock figure that will be painted
		 * Change 02 May 08:
		 * - only update the state and set the flag if need to redraw
		 * - move the draw function to other thread, so that it does not delay the time counting
		 */
		/*		public void actionPerformed(ActionEvent arg0) {
			// TODO Auto-generated method stub

			int totalTicks = quesDuration* 1000/SLEEP_TIME; // how many "ticks" in total, "tick" is each time the timer triger
			if (currentTick <totalTicks ) // count the current tick
			{
				currentTick = currentTick +1;
			} 
			else{
				//timer.stop();
				//endTime = System.currentTimeMillis();
				//System.out.println("Time duration in milliseconds: " + (endTime - startTime));
			}		

			// update the time string in cycle
			if (currentTick%timeStrUpdateCycle ==0 && currentTick/timeStrUpdateCycle <=quesDuration)
			{
				timeLeft = quesDuration-currentTick/timeStrUpdateCycle;
				timeString= Integer.toString(timeLeft);
				System.out.println("Time left, need to redraw: " + timeLeft);
				redrawTimeString = true;// need to redraw
			}
			if (currentTick%clockUpdateCycle ==0)
			{
				angleDraw = currentTick/clockUpdateCycle * angleUnit;	
				redrawClock = true;// need to redraw
			}						
		}//
		 */
		/**
		 * the Runnable interface
		 * this thread will update the GUI, the process take long time so need to run on separated thread
		 */
		public void run() {
			// TODO Auto-generated method stub
			while (true){
				if (redrawClock || redrawTimeString && (timeLeft!=0))
					paintMyGame();
				if (timeLeft ==0){	
					paintMyGame();
					endTime = System.currentTimeMillis();
					//System.out.println("Total time reding question: " + (endTime-startTime));
					break;
				}
			}
		}//

		public void paintMyGame(){							
			if (redrawClock ||redrawTimeString){
				if (redrawTimeString){
					if (timeLeft <=5 && timeLeft >0)
						playerClick.playSound();
					if (timeLeft==0)
						playerEndQues.playSound();
				}
				redrawClock =false;
				redrawTimeString = false;				
				repaint();
			}								
		}//

		//////////////////////////////////////////////////////////////////////////////
		private class MyTimer extends TimerTask{
			/**
			 * this function will be fired when the timer triger every 10 milliseconds
			 * it is responsible for update the time string (15,14,13,...0) and the area of the clock figure that will be painted
			 * Change 02 May 08:
			 * - only update the state and set the flag if need to redraw
			 * - move the draw function to other thread, so that it does not delay the time counting
			 */
			public void run() {								
				// TODO Auto-generated method stub
				// update the time string in cycle
				if (currentTick%timeStrUpdateCycle ==0 ){
					timeLeft = quesDuration-currentTick/timeStrUpdateCycle;
					//System.out.println("Time left, need to redraw: " + timeLeft);
					redrawTimeString = true;// need to redraw
				}
				if (currentTick%clockUpdateCycle ==0){
					angleDraw = currentTick/clockUpdateCycle * angleUnit;	
					redrawClock = true;// need to redraw
				}	

				if (currentTick <=totalTicks ) {// count the current tick
				
					currentTick = currentTick +1;
				} 
				else{
					// end the timer
					timer.cancel();
					endTime = System.currentTimeMillis();
					//System.out.println("The last tick " + currentTick + " total ticks " + totalTicks + " time left: " + timeLeft);
					//System.out.println("Time duration in milliseconds: " + (endTime - startTime));
				}							
			}//			
		}// End sub class MyTimer
	}//end class ClockWise
	////////////////////////////////////////////////////////////////////////////////////////////////////////////
	private class TextPanel extends JTextArea{

		private int topMargin;
		private int leftMargin;
		private int high, wide;
		int w;
		int h;
		public TextPanel(){
			super();
			initTextPanel();	
		}

		private void initTextPanel(){
			this.setBackground(Color.BLACK);			
			//LayoutManager myLayout = new BoxLayout(this, BoxLayout.Y_AXIS);
			//this.setLayout(myLayout);
			//this.setIgnoreRepaint(true);					
		}

		public void displayQuestion(Question myQuestion){		
			w = getWidth();
			h =getHeight();
			this.setLineWrap(true); // to wrap the text to the new line if the text is too long.

			Font f = new Font("Sanserif", Font.PLAIN,h/15);
			this.setFont(f);
			this.setForeground(Color.WHITE);

			Insets ins = new Insets(h/50, w/6, h/50, w/8);
			this.setMargin(ins);
			//display question text and alternatives
			this.setText(myQuestion.getQuestion());
			this.append("\n");
			this.append("\n");
			for (int i=0; i < myQuestion.getNumberOfAlt(); i++){
				this.append((i+1) + ". " + myQuestion.getAlternatives()[i] );
				this.append("\n");
			}			
			validate();
		}

	}// end class TextPanel



} // En class
