package Masterclient.core;

import java.io.IOException;
import java.util.ArrayList;

import Masterclient.Graphics.GameBoard;
import Masterclient.Graphics.SoPlayer;


public class Game {

	private MessagesExchangeThread masterClient; // client class used to communicate with game engine
	private Thread receiverThd; // Receiver thread, getting message from game engine
	private boolean gameCreatedOnServer;	
	private int lectureID;
	private int gameID;
	private int stateOfScn=-1;
	private GameBoard gameBoard;
	private Question currentQues;
	private Respondent currentAnsArray[];
	private ArrayList <Respondent> userLoginedList; // the name list of mobile player login
	SoPlayer PlayerAttention = new SoPlayer("resources/pause.wav");
	SoPlayer PlayerGameOver = new SoPlayer("resources/smas-smb3_game_over.wav");
	/**
	 * contructor
	 */
	public Game(){
		masterClient = new MessagesExchangeThread(this);
		receiverThd = new Thread(masterClient);
		gameCreatedOnServer =false;				
		gameBoard = new GameBoard(this);	
		userLoginedList = new ArrayList<Respondent>();
	}

	/**
	 * the entry for the game
	 */
	public static void main(String[] args){
		Game game = new Game();
		try {
			game.playGame();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			System.exit(1);
		}		
	}
	
	/**
	 * @throws IOException 
	 * 
	 */
	private void playGame() throws IOException
	{
		exchangeHandShake();
		login();
		setParameters();
		 // display the first screen
		// this will make a call to gameBoard.displayLoginIdle(gameID);
		 receiverThd.start();
		this.setStateOfScn(LG.SCREEN_STATE_LOGIN_IDLE); 		 
	}

	/**
	 * exchange handshake sign, it not correct then exit game
	 * @throws IOException 
	 *
	 */
	private void exchangeHandShake() throws IOException
	{
		masterClient.send(Message.HANDSHAKE_MASTER);
		String handshakeStr ="";
		handshakeStr = masterClient.receiveStr();
		if (!handshakeStr.equals(Message.HANDSHAKE_MASTER))
		{
			System.out.println("The server is not game engine!");
			System.exit(1);
		}
	}

	/**
	 * login function
	 * display login form, send information to server
	 * get the login result from server
	 * Should use some login management here, for example retrival forgot password, longin 3 times
	 * @throws IOException 
	 */
	private void login() throws IOException{
		String user ="tien", password = "game2008";
		int loginResult;
		String messageStr ="";
		// Display login form
		// Get the username and password provided in the login form
		// Send user, pass to Gamge Engine
		masterClient.send(user);
		masterClient.send(password);

		messageStr =masterClient.receiveStr();
		loginResult = Integer.parseInt(messageStr);
		if(loginResult !=LG.ERROR_LOGIN_OK)
		{
			System.out.println("Incorrect user name or password");
			System.exit(1);
		} else System.out.println("Well come " + user);

		messageStr = masterClient.receiveStr();
		
		gameID = Integer.parseInt(messageStr);
		if (gameID >=0){
			gameCreatedOnServer= true;
			System.out.println("Game has been created, set some parametter before you can play!" + gameID);
		} else {
			System.out.println("Game has not been created");
			System.exit(1);
		}		
	}

	/**
	 * Display from for user to set up some parameters
	 */
	private void setParameters()
	{
		if (!gameCreatedOnServer)
			return;
		System.out.println("Display a form to set up parameter at this time!");
		
		// From this time the messge send to server need to be follow a format, see document
		lectureID=1;
		 String msgLecID = Message.MSG_LECTURE_ID + Integer.toString(lectureID);
		 String msgMode = Message.MSG_GAME_MODE + Integer.toString(LG.GAMEMODE_BEAT_THE_SCORE);
		 
		 masterClient.send(msgLecID);
		 masterClient.send(msgMode);		 
	}//
	
	public void startGame(){
		// Start the game 
		// triger when user press button enter or space button)
		 String msgStart = Message.MSG_START + "";
		 masterClient.send(msgStart);
		
	}//
	
	/***
	 * Parses the input strings and calls the appropriate method with appopriate in/out.
	 * Called by the receiver thread when the message arrived from game engine
	 * @param msg: message from GE
	 */
	public void onReceive(Message msg) {
		//System.out.println("message receive from GE: " + msg.getRawMessage());

		if (msg.isQuestion())
		{
			currentQues = msg.buildQuestion();
			currentQues.printQuestion();
			PlayerAttention.playSound();
			gameBoard.displayQuestion(currentQues);
			setStateOfScn(LG.SCREEN_STATE_QUESTION_POSED);
		} else if(msg.isAnswer()){
			currentAnsArray = msg.getAnswerArray();
			gameBoard.displayAnswer(currentQues,currentAnsArray);
			//setStateOfScn(LG.SCREEN_STATE_DISPLAY_ANSWER);
		} else if (msg.isEndOfRound()){
			currentAnsArray = msg.getAnswerArray();
			PlayerGameOver.playSound();
			// chagege state to activate function gameBoard.displayEndOfRound(currentAnsArray);
			setStateOfScn(LG.SCREEN_STATE_END_ROUND);
			
		} else if (msg.isEndOfGame()){
			gameBoard.displayEndOfGame(null);			
		} else if(msg.isMobileLogin()){
			Respondent res = new Respondent(msg.getMsgContent(),0);
			userLoginedList.add(res);
			//msg.printMsg();
			gameBoard.repaint();
		}
	}//
	
	public void setStateOfScn(int s){
		stateOfScn=s;
		gameBoard.repaint();
	}
	
	public int getStateOfScn(){
		return stateOfScn;
	}
	
	public int getGameID(){
		return gameID;
	}
	
	public Respondent[] getAnswerArray(){
		return currentAnsArray;
	}
	
	public Question getCurrentQuesiton(){
		return currentQues;
	}
	
	public ArrayList<Respondent> getUserLogin(){
		return userLoginedList;
	}//
	
	public MessagesExchangeThread getMaster(){
		return masterClient;
	}
}
