package Masterclient.core;


/**
 * responsible for create general messages that are used in all modes.
 * the messages that are specific to mode will be created in GameMode class or child class 
 * @author tranlongtien
 *
 */
public class Message {
	protected char msgType;
	protected String msgContent;
	protected String msgRaw;
	
	public static final int MAX_NUMBER_OF_FIELDS_IN_MSG = 4000; // maximum number of unique fields in a message. do not change
    /**
     * These characters are prepended to messages that are sent to Game Engine, to show what type of messages they are
     * MSGPC: messages exchanged between PC client and game engine
     * MSGMB: messages exchanged between mobile and game engine
     */
	
	public static final char MSG_UNKNOWN_MESSAGE = '_';
	public static final char MSG_NUMBER_OF_QUES = '0';
	public static final char MSG_GAME_MODE = '1';
	public static final char MSG_START = '2'; // Start the game
	public static final char MSG_EXIT = '3'; // Exit game
	public static final char MSG_LECTURE_ID = '4'; // message to set lecture ID.
	public static final char MSG_END_OF_GAME = '5'; // message sent to client when reaching end of game.
	public static final char MSG_END_OF_ROUND = '6';
    
	public static final char MSG_QUESTION ='7';
	public static final char MSG_ANSWER ='8';
	
	public static final char MSG_LOGIN ='9';
	public static final char MSG_MOBILE_LOGIN ='A';
	public static final char MSG_NEXT_QUESTION ='B';
    /**
     * Handshake string. Each end of the connection sends this  string to the 
     * other just after the connection is opened.  This is done to confirm that 
     * the program on the other side of the connection is a proper program.
     */

    public static final String HANDSHAKE_MASTER = "MasterServer";
    public static final String HANDSHAKE_MOBILE = "MobileServer";
    
    public Message(String rawMessage)
    {
    	msgRaw = rawMessage;
    	msgContent = getMsgContent();
    	msgType = getTypeOfMsg();
    }

    
    public char getTypeOfMsg()
    {
    	char type;
    	if (msgRaw.length() >0)
    	{
    		type = msgRaw.charAt(0);
    			return type;
    	}
    	return MSG_UNKNOWN_MESSAGE;
    }//
    
    public String getMsgContent()
    {
    	if (msgRaw.length() >1)
    		return msgRaw.substring(1);
    	return null;
    }
    
    public String getRawMessage()
    {
    	return msgRaw;
    }
    
	public  void printMsg(){
		System.out.println("This message is printed from Message class: ");
		System.out.println ("msgraw: " + msgRaw + ", msgcontent: " + msgContent + ", type: " + msgType);		
	}

    
    public static String createExitMessage()
    {
    	String ret;
    	ret = Message.MSG_EXIT + "";
    	return ret;
    }
    
    public static String createEndGameMessage()
    {
    	String ret;
    	ret = Message.MSG_END_OF_GAME + "";
    	return ret;
    }

    public static String createStartGameMessage()
    {
    	String ret;
    	ret = Message.MSG_START + "";
    	return ret;
    }
    
    /**
     * analyse the formated question get from game engine and make the question.
     * @param msg
     */
    public Question buildQuestion()
    {
    		
    	   int numberOfFields = 0;
    	    String[] stringList = new String[Message.MAX_NUMBER_OF_FIELDS_IN_MSG];
    	    
    	    
    	    while (numberOfFields < Message.MAX_NUMBER_OF_FIELDS_IN_MSG) {
    	        int msgLength = msgContent.indexOf("|");
    	        if (msgLength == (-1)) {
    	            stringList[numberOfFields] = msgContent.substring(0, msgContent.length());
    	            numberOfFields++;
    	            break;
    	        } else {
    	            stringList[numberOfFields] = msgContent.substring(0, Math.abs(msgLength));
    	            msgContent = msgContent.substring(msgLength + 1, msgContent.length());
    	            //System.err.println(received + " " + msgLength);
    	            numberOfFields++;
    	        }
    	    }
    	 
         String question = stringList[0];
         int numberOfAlt = Integer.parseInt(stringList[1]);
        String[] alternatives = new String[numberOfAlt];
        for (int i = 0; i < numberOfAlt; i++) {
            alternatives[i] = (stringList[i + 2]);
        }
        String timeLimit = stringList[numberOfAlt + 2];
        int correctAlternativeIndex = Integer.parseInt(stringList[numberOfAlt + 3]);        
        return new Question(question,numberOfAlt,alternatives,timeLimit,correctAlternativeIndex);
    }
    
    /**
     * check if the message received is a question
     * 
     */
    public boolean isQuestion()
    {
    	if (msgRaw == null)
    		return false;
    	return (getTypeOfMsg() == Message.MSG_QUESTION);
    }
    
    /**
     * check if the message received is an answer
     * 
     */
    public boolean isAnswer()
    {
    	if (msgRaw == null)
    		return false;
    	return (getTypeOfMsg() == Message.MSG_ANSWER);
    }
    
    /**
     * get an array of answer (respond) is the message is an answer
     * example of message that is answer: "8|4|user1|0|user2|1|user3|2|user4|3"
     * @return
     */
    public Respondent[] getAnswerArray(){
    	String msg = msgContent;    	    	
    	// msg ="4|user1|0|user2|1|user3|2|user4|3";
    	int numberOfFields = 0;
    	int noUsers =0;
    	Respondent[] Answers;
   	    String[] stringList = new String[Message.MAX_NUMBER_OF_FIELDS_IN_MSG];
   	    
   	    
   	    while (numberOfFields < Message.MAX_NUMBER_OF_FIELDS_IN_MSG) {
   	        int msgLength = msg.indexOf("|");
   	        if (msgLength == (-1)) {
   	            stringList[numberOfFields] = msg.substring(0, msg.length());
   	            numberOfFields++;
   	            break;
   	        } else {
   	            stringList[numberOfFields] = msg.substring(0, Math.abs(msgLength));
   	         msg = msg.substring(msgLength + 1, msg.length());
   	            //System.err.println(received + " " + msgLength);
   	            numberOfFields++;
   	        }
   	    }
   	    noUsers = Integer.parseInt(stringList[0]);
   	    Answers = new Respondent[noUsers];
   	    for (int i=0; i< noUsers;i++){
   	    	String user =stringList[2*i+1];
   	    	int alternative = Integer.parseInt(stringList[2*i+2]);
   	    	Answers[i]= new Respondent(user,alternative);
   	    }
   	    
    	return Answers;
    }//
    
    /**
     * check if the message received is an answer
     * 
     */
    public boolean isEndOfRound()
    {
    	if (msgRaw == null)
    		return false;
    	return (getTypeOfMsg() == Message.MSG_END_OF_ROUND);
    }//
    
    /**
     * check if the message received is an answer
     * 
     */
    public boolean isEndOfGame()
    {
    	if (msgRaw == null)
    		return false;
    	return (getTypeOfMsg() == Message.MSG_END_OF_GAME);
    }//
    
    public boolean isMobileLogin(){
       	if (msgRaw == null)
    		return false;
    	return (getTypeOfMsg() == Message.MSG_MOBILE_LOGIN);
    }
    
}// End class
