import java.io.DataOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.PrintStream;

import javax.microedition.io.Connector;
import javax.microedition.io.SocketConnection;
import javax.microedition.lcdui.Command;
import javax.microedition.lcdui.Display;
import javax.microedition.lcdui.TextBox;

import com.sun.midp.io.BufferedConnectionAdapter;

/** 
 * Works in a producer consumer relationship to the grpical processing Thread. This means that when
 * graphical processing is finished and one waits for network input this Thread is woken, and parses the
 * following network message before releaving control to graphic processing and sleeps indefineltely.
 * Created by IntelliJ IDEA.
 * User: ofsdahl
 * Date: 10.jan.2007
 * : 15:29:56
 * To change this template use File | Settings | File Templates.
 */
public class MessagesExchangeThread implements Runnable {

    protected LeGaMidlet lega;
    protected OutputStream os;
    
    protected InputStream is;
    protected SocketConnection sc;
    protected int protocolState;
    protected boolean stringBuffered;
    public String received;
    protected Thread networkThread;
    protected boolean alive;

    /**
     * Initialize variables
     *
     * @param lega Midlet class
     */
    public MessagesExchangeThread(LeGaMidlet lega) {
        this.lega = lega;
        sc = null;
        is = null;
        os = null;
        protocolState = Constants.PROT_ENDED_STEP2;
        stringBuffered = false;
        received = null;
        networkThread = Thread.currentThread();
        alive = true;
        //////////////////////////////////////////////////////////////////////////////////
        boolean soFarSoGood = true;
        try {
            sc = (SocketConnection) Connector.open(Constants.SERVER_ADDRESS);
            sc.setSocketOption(SocketConnection.DELAY, 0);
            sc.setSocketOption(SocketConnection.KEEPALIVE, 30);
        } catch (Exception e) {
            soFarSoGood = false;
            lega.networkFailure();
        }


        try {
            is = sc.openInputStream();
            
        } catch (Exception e) {
            soFarSoGood = false;
            lega.networkFailure();
        }


        try {
            os = sc.openOutputStream();
        } catch (Exception e) {
            soFarSoGood = false;
            lega.networkFailure();
        }

        if (soFarSoGood)
            lega.networkStatus(true);
        else {
            lega.networkStatus(false);
            lega.networkFailure();
        }


        
    }//

    /**
     * Alternates between interpreting network input and sleeping. Loops forever.
     *
     *
     */
    public synchronized void run() {
        System.err.println("run receive thread!");
        while (alive) {
            received = receive();
            if (!(received==null)) {
                stringBuffered = true;
                try {
                    //lega.parse(received);
                	Message msg = new Message(received);
                	lega.onReceive(msg);
                    //wait(); //tientl
                } 
                //catch (InterruptedException e) {
                catch (Exception e) {
                  e.printStackTrace();
                }
            }

            try {
                Thread.currentThread().sleep(Constants.SAMPLE_RATE);
            } catch (InterruptedException e) {
                e.printStackTrace();

            }
        }// end while
    }//

    /**
     * Notify the object monitor that it is time to wake up and start processing network input.
     *
     */
    public synchronized void wakeMe() {
        notifyAll();
    }
    
    /**
     * receive a message from server
     * the end of message is new line character '\n' and it will be eliminated on the received string
     */
    public String receive(){
    	int ch =0;
    	String rec ="";
    	while(ch!=-1){
    		try{
    			ch = is.read();
    			char readChar = (char) ch;     		        	
    			rec = rec+ readChar;    			    			
    			if (readChar ==Message.MSG_END_MESSAGE){
    				break;        		
    			}
    		}catch(Exception e){
                e.printStackTrace();
                ch = -1;
                alive = false;
    		}
    	}//end while
    	rec = rec.trim();
    	if (rec.equalsIgnoreCase("A"))
    		rec="";    	
    	return rec;
    }
    
    Message receiveMsg(){
    	String strReceive = receive();
    	Message msg = new Message(strReceive);
    	return msg;
    }

    /**
     * Strips the prtocol header and signal chars off the message.
     *
     * @return
     */
/*    public String receiveStr() {
        String rec = "";
        int ch = 0;
        char[] tempChar = new char[1];
        while (ch != -1) {
            try {
                ch = is.read();
                
                //System.err.print(ch);
                tempChar[0] = (char) ch;
                String tempString = new String(tempChar);
                rec = rec + "" + new String(tempChar);
                if ((protocolState == Constants.PROT_ENDED_STEP2)) {
                    if (tempString.equals("@")) {
                        Display.getDisplay(lega).flashBacklight(10);
                        protocolState = Constants.PROT_STARTED_STEP1;
                    } else if(tempString.equals("A")){
                        Display.getDisplay(lega).flashBacklight(10);
                         rec = "";
                        break;
                    }    else {

                        rec = "";
                        break;
                    }
                } else if ((protocolState == Constants.PROT_STARTED_STEP1)) {
                    if (tempString.equals("^")) {
                        protocolState = Constants.PROT_STARTED_STEP2;
                    } else {
                        rec = "";
                        break;
                    }
                } else if (protocolState == Constants.PROT_STARTED_STEP2) {
                    if (tempString.equals("^")) {
                        protocolState = Constants.PROT_ENDED_STEP1;
                    }
                } else if ((protocolState == Constants.PROT_ENDED_STEP1)) {
                    if (tempString.equals("@")) {
                        protocolState = Constants.PROT_ENDED_STEP2;
                        break;
                    } else {
                        protocolState = Constants.PROT_STARTED_STEP2;
                    }
                }


            } catch (IOException e) {
                e.printStackTrace();
                ch = -1;
                alive = false;
            }

        }
        System.err.println(rec);
        if (rec.length() > 3) {
            return rec.substring(2, (rec.length() - 2));
        }
        return null;
    }*/

    /**
     * Closes connections from the client side.
     *
     * @throws IOException
     */
    public void close() throws IOException {
    	send(Message.MSG_EXIT+"");
        os.close();
        is.close();
        sc.close();
    }

    /**
     * Sends the provided String to the server.
     *
     * @param str
     */
    public void send(String str) {
        try {
        	str = str + Message.MSG_END_MESSAGE;
            os.write(str.getBytes());
            os.flush();
            System.err.println("Sender " + str);
        } catch (IOException e) {
            e.printStackTrace();
        }
    }
}
